
  /************************************************************************/
  /****   Copyright (C) 1991, 1992, 1993, 1994, 1995 by Brian Reed.    ****/
  /****   All rights reserved.  Written by Brian Reed.                 ****/
  /************************************************************************/
  /****   Brian Reed, 29328 Fairfield, Warren MI  48093.  73503,3364   ****/
  /************************************************************************/
  /****   This is the main include file for the BF library.            ****/
  /************************************************************************/


/********  Protect against multiple inclusion.  ********/
#ifndef BF_H
#define BF_H


/********  Define bfDEMO compilation modes.  ********/

  /*  Set to 1 to 'bust' character display for demo versions.  */
  #define bfDEMO     1

  /*  Set to 1 to exclude MemCheck.  */
  #define bfNOMEMCHECK  1


/********  Include the platform independent header info.  ********/

  /****  Get the platform independent stuff.  ****/
  #include "bfplat.h"


/********  Generic 'platform' defines.  ********/

  /****  Define an memory model identifier.                 ****/
  /****  Can pre-define this in the platform-specific .H.   ****/
  #ifndef bfFAR
    #define bfFAR
  #endif

  /****  A type for use in the external lib customized per gfx engine.  ****/
  /****  >>Should<< pre-define this in the platform-specific .H.        ****/
  #ifndef bfBitMapPtr
    typedef void bfFAR *bfBitMapPtr;
  #endif

  /****  Define an internal color variable.                 ****/
  /****  Can pre-define this in the platform-specific .H.   ****/
  #ifndef bfColor
    typedef unsigned long bfColor;     /*  Internal color def.  */
  #endif

  /****  An alternate for NULL.  bfNULL must equal 0.      ****/
  /****  Can pre-define this in the platform-specific .H.  ****/
  #ifndef bfNULL
    #define bfNULL       (void *)0
  #endif

  /****  Define function calling type.                      ****/
  /****  Can pre-define this in the platform-specific .H.   ****/
  #ifndef bfDECL
    #define bfDECL
  #endif

  /****  Define fCODE, which for some platforms affects     ****/
  /****  static data storage (data in the code segment).    ****/
  /****  Can pre-define this in the platform-specific .H.   ****/
  #ifndef bfCODE
    #define bfCODE
  #endif

  /****  Define fSCREEN_Y_FLIPPED which is used in bottom-  ****/
  /****  up screen displays.                                ****/
  /****  Changing this requires recompiling the BF lib.     ****/
  #ifndef bfSCREEN_Y_FLIPPED
    #define bfSCREEN_Y_FLIPPED 0
  #endif

  /****  Define RGB colors.                                 ****/
  /****  Can pre-define these in the platform-specific .H.  ****/
  #ifndef bfWHITE
    #define bfWHITE     0x00FFFFFFL
    #define bfLIGHTGRAY 0x00C0C0C0L
    #define bfGRAY      0x00808080L
    #define bfDARKGRAY  0x00505050L
    #define bfBLACK     0x00000000L
    #define bfRED       0x00FF0000L
    #define bfGREEN     0x0000FF00L
    #define bfBLUE      0x000000FFL
    #define bfYELLOW    0x00FFFF00L
    #define bfCYAN      0x0000FFFFL
    #define bfPURPLE    0x00FF00FFL
    #define bfGOLD      0x00FFD000L
  #endif


  /****  DOS DEBUG mode flags.  ****/
  #define CURSORDEBUG    0
  #define FDEBUG         0
  #define FBFDEBUG       0
  #define FBFDEBUGLST    0

  /****  General return values.  ****/
  #define bfSUCCESS       1      /* General success.  */
  #define bfFAILURE       0      /* Gnrl err. NEEDS to match bfNULL! */

  /****  Errors  ****/
  #define bfNOERROR          (SWORD)0
  #define bfNOMEMORY         (SWORD)-1
  #define bfNOFILE           (SWORD)-2
  #define bfBADTYPE          (SWORD)-3
  #define bfREADERR          (SWORD)-4
  #define bfWRITEERR         (SWORD)-5
  #define bfPROTECTEDMODE    (SWORD)-6
  #define bfNOINIT           (SWORD)-7
  #define bfBADFONT          (SWORD)-8
  #define bfBADSTATE         (SWORD)-9
  #define bfBADFILE          (SWORD)-10
  #define bfTOOBIG           (SWORD)-11
  #define bfBADPARAM         (SWORD)-12
  #define bfUNKNOWNERR       (SWORD)-13  /* Last error.  KEEP THESE TWO    */
  #define bfERR_MSG_COUNT            13  /* THE SAME for fGetErrorString() */

  /****  Length of the static work buffers, etc.  ****/
  #define bfWORKLEN        1024+8
  #define bfDIRLEN          128

  /****  Radians conversion.  ****/
  #define bfRAD(x) ((x) * 3.1415927 / 180.0)


/********  Font defines  ********/

  /*  GFP library must MATCH this in gMode!!  */
  typedef enum {              /****  Display modes  ****/
    /*bf4BIT       =  0,          maybe someday. */
    bf8BIT       =  1,            /*  These must stay at   */
    bf15BIT      =  2,            /*  BYTESPERPIXEL value! */
    bf24BIT      =  3,
    /*bf32BIT      =  4,          maybe someday. */
  } bfMode;

  typedef enum {              /****  Error actions.  ****/
    bfCLEAR  = 1,
    bfKEEP   = 2,
  } bfErrorClear;

  typedef enum {              /****  Font direction  ****/
    bfTEXTRIGHT = 1,
    bfTEXTDOWN  = 2
  } bfDir;

  typedef enum {              /****  Font modification  ****/
    bfNORMAL = 1,
    bfUNDER  = 2,
    bfBOLD   = 4,
    bfDROP   = 8,
    bfBACK   = 16,
  } bfFace;

  typedef enum {              /****  Horiz justifications  ****/
    bfHLEFT   = 1,
    bfHRIGHT  = 2,
    bfHCENTER = 4
  } bfHorizJust;

  typedef enum {              /****  Vert justifications  ****/
    bfVTOP     = 1,
    bfVBOTTOM  = 2,
    bfVCENTER  = 4,
    bfVBASE    = 8
  } bfVertJust;

  typedef enum {                 /****  Font types  ****/
    bfINVALIDFONT  = 0,
    bfGEMFONT      = 1,
    bfBMFFONT      = 2,
    bfTTFONT       = 3,  /* only in Windows currently. */
  } bfType;

  typedef enum {                 /****  Pixel draw modes  ****/
    bfPUT           = 1,          /*  MUST MATCH bfBackMode!   */
    bfTRANS         = 2,          /*  MUST MATCH bfBackMode!   */
    bfBLEND         = 4,
    bfIMAGE         = 8,
    /* bfALPHA = 16, */  /*  for over vid.. maybe f32BIT specifies same?  */
  } bfDrawMode;

  typedef enum {
    /* Dont use any that overlap from fDrawMode!?  */
    bfBACKSOLID     = 0,
    /* bfBACKTILE      = 2, */
    /* bfBACKFADEEDGELEFT  = ,*/
    /* bfBACKFADEEDGERIGHT  = ,*/
    /* bfBACKFADEEDGETOP  = ,*/
    /* bfBACKFADEEDGEBOTTOM  = ,*/
    bfBACKBORDER    = 64,
  } bfBackStyle;

  typedef enum {                 /*****  BF Data types  *****/
    bfBOTH   = 0,                /*  For opening outline fonts like TTs.  */
    bfRAW    = 1,
    bfRLE    = 2
  } bfCompress;

  typedef enum {                 /****  Passed to gfx engine  ****/
    bfGFXLITERAL   = 0,          /*  Defines data passed to gfx. */
    bfGFXAA        = 1
  } bfGFXType;

  typedef struct {               /****  Kerning info.  ****/
    UWORD first;                 /*  Dont change size without modifying  */
    UWORD second;                /*  Dont change size without modifying  */
    SWORD kerndistX;             /*  the OpenTT() code.                  */
    /*SWORD kerndistY;*/         /*  (not yet implemented)               */
  } bfKernPair;   /* Kdist is added to cell size, so usually is negative. */

  #if 0
  FUTURE STUFF
  /*  used for font input function  */
  typedef enum {              /****  bkgd solid color or bitmap  ****/
    /bfSOLIDBACK = 1,  change these names when incorporated
    /bfIMAGEBACK = 2   they're too similar to bfBackStyle
  } bfBackMode;

  /*  used for font input function  */
  typedef enum {              /****  Insert or overwrite text  ****/
    bfINS = 1,
    bfOVR = 2
  } bfInsertMode;
  #endif


  /*  2 char ID for GEM font files..  */
  #define bfGEMFILEID  ( 0U )

  /****  GEM font file header struct.  ****/
  typedef struct {
    SWORD  id;                  /* font id                    */
    SWORD  pt_size;             /* point size                 */
    SBYTE  name[32];            /* name of font               */
    SWORD  min;                 /* min char ascii value       */
    SWORD  max;                 /* max char ascii value       */
    SWORD  top;                 /* top line distance          */
    SWORD  ascent;              /* ascent distance            */
    SWORD  half;                /* half line distance         */
    SWORD  descent;             /* descent distance           */
    SWORD  bot;                 /* bottom line distance       */
    SWORD  char_width;          /* max char width             */
    SWORD  cell_width;          /* max cell size              */
    SWORD  left_offset;         /* left offset (not used)     */
    SWORD  right_offset;        /* right offset (not used)    */
    SWORD  thicken;             /* thicken amt in pixels      */
    SWORD  ul_thick;            /* underline thickness        */
    SWORD  light_mask;          /* lightening mask (5555h)    */
    SWORD  skew_mask;           /* skew mask (5555h)          */
    SWORD  flags;               /* flags                      */
    SLONG  hot_ptr;             /* horz offset table ptr      */
    SLONG  cot_ptr;             /* char offset table ptr      */
    SLONG  buf_ptr;             /* offset to data buffer      */
    SWORD  form_width;          /* sum of char widths (bytes) */
    SWORD  form_height;         /* height in raster lines     */
  } bfGEMheader;


  /*  4 char ID as 'BMF:', the hard way..  */
  #define bfBMFFILEID  ( (((long)'M')<<8L)     \
                        +((long)'B')           \
                        +(((long)':')<<24L)    \
                        +(((long)'F')<<16L) )

  typedef enum {                    /*****  BMF Font types  *****/
    bfBMF16AA   = 1,                 /*  16 level anti-aliasing.           */
    bfBMF256AA  = 2, /*unsupported*/ /*  256 level anti-aliasing.          */
    bfBMF16AAO  = 3, /*unsupported*/ /*  16 level fore, 16 level outline.  */
    bfBMF16CLR  = 4  /*unsupported*/ /*  16 color font with pal.           */
  } bfBMFType;

  /****  BMF font char struct.  ****/
  typedef struct {
    SWORD        boxX;          /* Size of char bitmap                  */
    SWORD        boxY;          /* Size of char bitmap                  */
    SWORD        boxDescent;    /* Baseline distance (for Y offset)     */
    SWORD        positionX;     /* Left offset at which to draw bits.   */
    SWORD        cellincX;      /* Cell increment to next cell          */
    bfCompress   datatype;      /* RAW or RLE data                      */
    UBYTE  bfFAR *pixels;       /* Pointer to THIS chars bits           */
  } bfBMFFontChar;

  /****  BMF font file char struct used for loading and saving.  ****/
  typedef struct {
    SWORD      thischr;           /* This char                           */
    SWORD      boxX;              /* Size of char bitmap in X.           */
    SWORD      boxY;              /* Size of char bitmap in Y.           */
    SWORD      boxDescent;        /* chars baseline to bottom distance   */
    SWORD      positionX;         /* Left offset at which to draw bits.  */
    SWORD      positionY;         /* Top offset to top of char bitmap.   */
    SWORD      cellincX;          /* Cell increment to next cell         */
    SWORD      cellincY;          /* Cell increment in Y.                */
    bfCompress datatype;          /* RAW or RLE data                     */
    ULONG      coffset;           /* THIS chars offset in bitmap         */
  } bfBMFFileChar;

  /****  BMF font file header struct.  ****/
  /****  Used kind of funny... This does not represent the complete      ****/
  /****  header in the file.  This struct is used for the runtime data.  ****/
  typedef struct {
    SBYTE               name[33];          /* name of font - ZERO padded   */
    SBYTE               eof;               /* terminate a DOS 'TYPE' here. */
    SWORD               pt_size;           /* point size                   */
    SWORD               version;           /* font file version #          */
    bfBMFType           type;              /* font type                    */
    SWORD               char_count;        /* count of chars in file       */
    SWORD               max_boxX;          /* max char width               */
    SWORD               max_boxY;          /* max char height              */
    SWORD               max_boxDescent;    /* max descent of char set      */
    SWORD               max_boxAscent;     /* max ascent of char set       */
    SWORD               ul_dist;           /* underline distance           */
    SWORD               ul_thick;          /* underline thickness          */
    SWORD               spacing;           /* font spacing.                */
    ULONG               bitmap_bytes;      /* bitmap alloc size            */
    UBYTE               color_palette[48]; /* 16 RGB values                */
    ULONG               user_data;         /* file offset todata at EOFont */
    UWORD               BMF_kerncount;     /* count of kern pairs in file  */
    UWORD               reserved1;         /* language flag maybe?         */
    bfBMFFontChar bfFAR *chars;            /* fBMFFontChar's               */
    UBYTE         bfFAR *bitmap;           /* pointer to char pixel data.  */
    bfMEMHANDLE         Hchars;            /* Used by some implementations.*/
    bfMEMHANDLE         Hbitmap;           /*  Both MUST=0 in font file!   */
  } bfBMFFontHeader;
  /* max_height is NOT total of max_desc + max_asce!  it is the pixel */
  /* height of the tallest char.  It may or may not extend to         */
  /* EITHER the asc or desc max.                                      */

  /*  KERN - Kern pair struct's (determine size from count)       */
  /*         Immediately following the bitmap, before user_data.  */

  /* USERDATA:  4 bytes as file offset to USERDATA                 */
  /*  Userdata will begin with another fBMFFILEID (4 bytes).       */
  /*  Following this ID will be a chunk name and size (4bytes ea). */
  /*   This chunksize value is the total count of bytes in this    */
  /*   chunk, not counting these 8 bytes themselves.               */
  /*  Predefined chunk types are:                                  */
  /*   256C - 256 color palette                                    */

  /****  GEM font struct.  ****/
  typedef struct {
    bfGEMheader        header;    /*  Font header.         */
    SWORD        bfFAR *cot;      /*  Font COT.            */
    UBYTE        bfFAR *bm;       /*  Font bitmap.         */
    bfMEMHANDLE        Hcot;      /*  Used by some memory  */
    bfMEMHANDLE        Hbm;       /*   implementations.    */
  } bfGEMFont;

  /****  bfData data struct.  ****/
  typedef union {
    bfBMFFontHeader    BMFFont;
    bfGEMFont          GEMFont;
  } bfData;

  /****  bfFont data struct.  ****/
  typedef struct bfFont_ {
    struct bfFont_ bfFAR *next;         /*  Next font.             */
    struct bfFont_ bfFAR *prev;         /*  Prev font.             */
    bfType               type;          /*  Font Type.             */
    bfData               data;          /*  Data per font type.    */
    UWORD                kerncount;     /*  Number of kern pairs.  */
    bfKernPair     bfFAR *kernpairs;    /*  Kern pair data.        */
    bfMEMHANDLE          Hfont;         /*  Used by some memory    */
    bfMEMHANDLE          Hkernpairs;    /*   implementations.      */
  };
  typedef struct bfFont_ bfFont;


  /****  bfState struct.  ****/
  typedef struct {
    bfMEMHANDLE          Hstate;               /*  Handle to itself.    */
    SWORD                id;                   /*  Hold state ID here.  */
    /*  Parameters.  */
    bfFont         bfFAR *font;                /*  Current font.        */
    SWORD                x, y;                 /*  Current position     */
    SWORD                hspacing;             /*  Current spacing.     */
    SWORD                vspacing;             /*  Current spacing.     */
    bfDir                direction;            /*  Current font dir.    */
    bfHorizJust          hjust;                /*  Current horiz just.  */
    bfVertJust           vjust;                /*  Current vert just.   */
    bfFace               face;                 /*  Current face.        */
    SWORD                ul_dist;              /*  Current UL distance. */
    SWORD                ul_thick;             /*  Current UL thickness.*/
    /*  Colors.  */
    bfColor              fore;                 /*  RGB fore color.      */
    bfColor              back;                 /*  RGB back color.      */
    bfColor              drop;                 /*  RGB drop color.      */
    UBYTE                fore256;              /*  256 color fore.      */
    UBYTE                back256;              /*  256 color back.      */
    UBYTE                drop256;              /*  256 color drop.      */
    bfColor              AAgradation[16];      /*  AA Fore->Back grad.  */
    bfColor              AAdropgradation[16];  /*  AA Drop->Back grad.  */
    UBYTE                AAgamma[16];          /*  AA gamma curve.      */
    UBYTE                AAindicies[16];       /*  AA indicie table.    */
    /*  Modes.  */
    bfDrawMode           drawmode;             /*  Font drawing mode.   */
    bfDrawMode           dropmode;             /*  DropShadow mode.     */
    SWORD                dropdistx;            /*  DropShadow X dist.   */
    SWORD                dropdisty;            /*  DropShadow Y dist.   */
    bfDrawMode           backmode;             /*  Back >256 mode.      */
    SWORD                backaddedL;
    SWORD                backaddedT;
    SWORD                backaddedR;
    SWORD                backaddedB;
    bfBackStyle          backstyle;            /*  Background style.    */
    SWORD                backstylesize;
    /*  Bitmaps.  */
    bfBitMapPtr          dest_bm;              /*  Font destination.    */
    bfBitMapPtr          image;                /*  Image to key to fore.*/
    UWORD                imagex;               /*  Key offset X.        */
    UWORD                imagey;               /*  Key offset Y.        */
    /* for dynamic (like Adobe Type 1) fonts                         */
    /*  float/double size, rotation, aspect, Xscale, Yscale, beta, gamma?  */
  } bfState;



/********  Global stuff.  ********/

  typedef struct {
    bfState bfFAR *font_state;                /* Current state.    */
    bfFont  bfFAR *font_first;                /* Pointer to 1st.   */
    bfState       fstate;                     /* Default state.    */
    bfMode        font_current_mode;          /* Graphic mode.     */
    SWORD         font_installed;             /* Installed flag.   */
    SWORD         font_open_count;            /* Font open count.  */
    SWORD         font_last_error;            /* Last error.       */
    UWORD         font_in_strdrawN;           /* Flag StrDrawN     */
    UWORD         font_in_strdrawN_horiz;     /* Flag no ul twice. */
    UWORD         font_in_strdrawN_lastchar;  /* Flag last char.   */
    UBYTE         font_dir[bfDIRLEN];         /* Font Directory.   */
    UBYTE         font_work[bfWORKLEN];       /* Temp buffer.      */
  } bfGlobalsStruct;

  /****  Make the global structure available.  ****/
  extern bfGlobalsStruct bfCODE bfGlobals;

  /**** Some default definitions. First is a shortcut to current state. ****/
  #define bfSTATE      (bfGlobals.font_state)
  #define bfSCREEN                     bfNULL
  #define bfSTATEID                    0x735A

  /******  These are defaults used by the gradate, for BF fonts.   ******/
  /******  NOTE! Gamma  2.20 = NTSC gamma per Roy Hall pg121.      ******/
  #define bfBETA                         1
  #define bfNTSCGAMMA            (REAL)2.2
  #define bfSVGAGAMMA            (REAL)1.7
  #define bfGAMMA              bfSVGAGAMMA



  /********  Main function prototypes.  ********/
  #include "bfproto.h"

  /********  MemCheck needs to be the last included file!  ********/
  #if (!bfNOMEMCHECK)
    #include <memcheck.h>
  #endif



#endif


