
/***********************************************************************/
/****   Copyright (C) 1991, 1992, 1993, 1994, 1995 by Brian Reed.   ****/
/****   All rights reserved.  Written by Brian Reed.                ****/
/***********************************************************************/
/****   Platform independent definitions.  Called by BF.H.          ****/
/***********************************************************************/

#ifndef BFPLAT_H
#define BFPLAT_H




/********  Set a default platform.  The user may define  ********/
/********  this before including BF.H.                  ********/
#ifndef bfDOS
#ifndef bfWIN
#ifndef bfGRASP
  /****  If its not a Windows compile, then DOS is the default.  ****/
  #ifdef _Windows
  # define bfWIN
  #else
  # define bfDOS
  #endif
#endif
#endif
#endif

/********  Make some typedefs for help in platform independence.  ********/

  /****  The regular types.  ****/
  typedef unsigned short     UWORD;
  typedef signed short       SWORD;
  typedef unsigned long int  ULONG;
  typedef signed long int    SLONG;
  typedef unsigned char      UBYTE;
  typedef signed char        SBYTE;
  typedef double             REAL;         /* Compiler default 'float'. */
  typedef signed short       bfMEMHANDLE;  /* Used as memory handle.    */



/****  Start of bfDOS.  ****/
#ifdef bfDOS

  #define bfCODE
  #define bfFAR               _far
  #define bfHUGE              _huge
  #define bfDECL              _pascal
  #define bfDECLNOEXPORT      _pascal
  #define bfSCREEN_Y_FLIPPED  0

  /****  Make an RGB triplet color type.  ****/
  typedef struct {
    UBYTE r,g,b;
  } bfRGB;



/****  End of bfDOS, start of fWIN.  ****/
#else
#ifdef bfWIN

  #include <windows.h>

  #define bfCODE
  #define bfFAR               _far
  #define bfHUGE              _huge
  // Gary says try these for a little speedup
  //#define bfDECL              FAR PASCAL _loadds
  #define bfDECL              FAR PASCAL _export
  #define bfDECLNOEXPORT      FAR PASCAL
  #define bfSCREEN_Y_FLIPPED  0
  #ifndef STRICT
    #define STRICT
  #endif
  #ifndef WIN31
    #define WIN31
  #endif

  /****  Make an RGB triplet color type.  ****/
  typedef struct {
    UBYTE b,g,r;
  } bfRGB;

  typedef COLORREF bfColor;

  #define bfWHITE     0x00FFFFFFL
  #define bfLIGHTGRAY 0x00C0C0C0L
  #define bfGRAY      0x00808080L
  #define bfDARKGRAY  0x00505050L
  #define bfBLACK     0x00000000L
  #define bfRED       0x000000FFL
  #define bfGREEN     0x0000FF00L
  #define bfBLUE      0x00FF0000L
  #define bfYELLOW    0x0000FFFFL
  #define bfCYAN      0x00FFFF00L
  #define bfPURPLE    0x00FF00FFL
  #define bfGOLD      0x0000D0FFL

  #define bfRAD(x) ((x) * 3.1415927 / 180.0)

  // BGR 12/94
  // its really a CPP header, and our code is really C code, but I compile
  // it as if CPP, for the added error checking.  Hence, extern as "C".
  // BC dependent?
  #ifdef __cplusplus
    extern "C" {
  #endif
      int bfDECL bfMemCheck_Start(void);
      int bfDECL bfMemCheck_End(void);
  #ifdef __cplusplus
    }
  #endif

  #if 0

  cut from BMF.C fActInit() !!
  fGlobals.font_state->image = (HDC)0;
  fGlobals.font_state->dest_bm = (HDC)0;
  fGlobals.font_state->dest_winXsize = 0L;
  fGlobals.font_state->dest_winYsize = 0L;

  /****  Memory allocation stuff.  'Huge' is for >64K allocs.  ****/
  #define fMalloc(s)      GlobalAlloc(GPTR, (DWORD)s)
  #define fHugeMalloc(s)  GlobalAlloc(GPTR, (DWORD)s)
  #define fFree(m)        GlobalFree(m)
  #define fHugeFree(m)    GlobalFree(m)

  int fAllocHuge(int *handle, char fFAR **ptr, long size) {
    if ((*handle=GlobalAlloc(size)) == 0) {
      *ptr = NULL;
      return FAILURE;
    }
    if ((*ptr=(char fFAR *)GlobalLock(*handle)) == NULL) {
      GlobalFree(*handle);
      *handle = 0;
      return FAILURE;
    }
    return SUCCESS;
  }

  stuff noted from BMF.H:

    font_first in fGlobals should be maintained?

    /* these next two NOT used by current windows implementation */
    /* could be, with the new memory scheme. */
    typedef struct fFont_ {
      fFont_ fFAR *next;         /*  Next font.             */
      fFont_ fFAR *prev;         /*  Prev font.             */

  #endif


/****  End of bfWIN, start of bfGRASP.  ****/
#else
#ifdef bfGRASP

GRASP

  #define bfCODE              _based(_segname("_CODE"))
  #define bfFAR               _far
  #define bfHUGE              _huge
  #define bfDECL              _pascal
  #define bfDECLNOEXPORT      _pascal
  #define bfSCREEN_Y_FLIPPED  1

  /****  Make an RGB triplet color type.  ****/
  typedef struct {
    UBYTE r,g,b;
  } bfRGB;

  /********  Tell CPP that we're C functions.  ********/
  #ifdef __cplusplus
    extern "C" {
  #endif

  /*  Grasp memory stuff.  */
  typedef struct {
    fMEMHANDLE (fFAR *allocseg)(SLONG size);
    void (fFAR *reallocseg)(fMEMHANDLE hand, SLONG size);
    void (fFAR *freeseg)(fMEMHANDLE hand);
    void fFAR * (fFAR *openseg)(fMEMHANDLE hand);
    void (fFAR *touchseg)(fMEMHANDLE hand);
    void (fFAR *closeseg)(fMEMHANDLE hand);
  } graspmemfuncs;

  #ifdef __cplusplus
    }
  #endif


  // Grasp stuff from fGlobals
    //#if (fGRASP)
    //  graspmemfuncs    *mem;  /*  Structure of memory funcs for Grasp.  */
    //#endif

  // from open.c
  //#if (!fGRASP)
  // #include <alloc.h>   /*  for allocation  */
  //#endif


#endif  // bfGRASP
#endif  // bfWIN
#endif  // bfDOS


/****  End of BFPLAT.H  ****/
#endif



