
/**  WINC.C  - A BorlandC / Win16 BeautiFonts (tm) demo.             **/
/**  Copyright 1994 & 1995 (c) by Brian Reed.  All Rights Reserved.  **/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "winc.h"       /*  This applications include file.  */
#include "dialog.h"     /*  This applications dialogs.  */
#include "bf.h"         /*  BF.H includes windows.h and memcheck.h.  */


extern HINSTANCE _hInstance;    /*  The global in the BC startup code.  */


/*  Draw the background.  */
void BackgroundDraw(HDC scrhdc, int flag)
{
static BITMAP bg_bmobj1, bg_bmobj2;
static HBITMAP bg_bm1=0, bg_bm2=0;
HBITMAP oldhbm;
HDC bmdc;
int x, y, W, H;

  /*  If flag == 0, then free the BMPs already loaded.  */
  if (flag == 0) {
    if (bg_bm1)
      DeleteObject(bg_bm1);
    bg_bm1 = 0;
    if (bg_bm2)
      DeleteObject(bg_bm2);
    bg_bm2 = 0;
    return;
  }

  /*  Check the destination and get its size.  */
  if (scrhdc == 0)
    return;
  W = GetDeviceCaps(scrhdc, HORZRES);
  H = GetDeviceCaps(scrhdc, VERTRES);

  /*  FLOCK.BMP  */
  if (flag == 1) {
    if (bg_bm1 == 0) {
      bg_bm1 = LoadBitmap(_hInstance, "BACK1");
      GetObject(bg_bm1, sizeof(BITMAP), &bg_bmobj1);
    }
    if (bg_bm1 != 0) {
      if ((bmdc=CreateCompatibleDC(scrhdc)) != NULL) {
        if ((oldhbm=(HBITMAP)SelectObject(bmdc, bg_bm1)) != 0) {
          for (x=0; x<W+bg_bmobj1.bmWidth; x+=bg_bmobj1.bmWidth)
            for (y=0; y<H+bg_bmobj1.bmHeight; y+=bg_bmobj1.bmHeight)
              BitBlt(scrhdc, x, y,
                     bg_bmobj1.bmWidth, bg_bmobj1.bmHeight,
                     bmdc, 0, 0, SRCCOPY);
          SelectObject(bmdc, oldhbm);
        }
        DeleteDC(bmdc);
      }
    }
  }

  /*  CASTLE.BMP  */
  if (flag == 2) {
    if (bg_bm2 == 0) {
      bg_bm2 = LoadBitmap(_hInstance, "BACK2");
      GetObject(bg_bm2, sizeof(BITMAP), &bg_bmobj2);
    }
    if (bg_bm2 != 0) {
      if ((bmdc=CreateCompatibleDC(scrhdc)) != NULL) {
        if ((oldhbm=(HBITMAP)SelectObject(bmdc, bg_bm2)) != 0) {
          for (x=0; x<W+bg_bmobj2.bmWidth; x+=bg_bmobj2.bmWidth)
            for (y=0; y<H+bg_bmobj2.bmHeight; y+=bg_bmobj2.bmHeight)
              BitBlt(scrhdc, x, y,
                     bg_bmobj2.bmWidth, bg_bmobj2.bmHeight,
                     bmdc, 0, 0, SRCCOPY);
          SelectObject(bmdc, oldhbm);
        }
        DeleteDC(bmdc);
      }
    }
  }
}


void cm_About(HWND hwnd)
{
HDC scrhdc;
LOGFONT lf;
HFONT hMyFont;
HFONT hfOld;
HCURSOR hcurSave;
int Xleft=28, y=32, i;
const char far *text[] = {
 "This is a demo of my Windows DLL anti-aliased font engine.  Anti-aliased fonts",
 "are used to reduce the 'jaggies' that appear in standard monochrome fonts.",
 "Be sure to try the TrueType 'Comparison Test' for a side-by-side view.",
 "",
 "For best viewing, run this demo at a resolution of 640x480 in an RGB display",
 "mode (32768, 65536 or 16 million colors).  256 color mode usually works,",
 "depending on the driver.",
 "",
 "The purpose of this demo is to evaluate user interest towards a potential",
 "product and all feedback is appreciated.  My font engine also has unfinished",
 "DOS and Grasp libraries.",
 "",
 "Next on my to-do list are a few cleanup's, speed optimizations, and underlines.",
 "This screen is not anti-aliased.",
 "",
 "",
 "Brian Reed  29328 Fairfield  Warren, MI  48093   73503.3364@compuserve.com",
 NULL
 };

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  Rectangle(scrhdc, 0,0, 32000,32000);

  /*  Print using regular TrueType font.  */
  GetObject(GetStockObject(SYSTEM_FONT), sizeof(lf), &lf);
  lf.lfHeight = 20;
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_LIGHT;
  _fstrcpy((char far *)&lf.lfFaceName, (char far *)"Tms Rmn");
  if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
    i=0;
    while (text[i]) {
      TextOut(scrhdc, Xleft, y, text[i], strlen(text[i]));
      y += lf.lfHeight;
      i++;
    }
    SelectObject(scrhdc, hfOld);
    DeleteObject(hMyFont);
  }
  else
    MessageBox(NULL, "Couldn't open Times New Roman", "Error", MB_OK);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_BMF_RGB(HWND hwnd)
{
bfFont *bmf;
HDC scrhdc;
HCURSOR hcurSave;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  if ((bmf=bfActOpen("ARIAL032.BMF")) == bfNULL) {
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);
    return;
  }

  BackgroundDraw(scrhdc, 1);
  bfSetFont(bfNULL, bmf);
  bfSetDrawMode(bfNULL, bfBLEND, 255);
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL,0,0, bfWHITE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,45,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfLIGHTGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,80,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,115,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfDARKGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,150,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfBLACK, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,185,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfRED, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,220,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfGREEN, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,255,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfBLUE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,290,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfYELLOW, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,325,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfCYAN, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,360,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfPURPLE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 190,395,"Text Line For Test!");

  bfActClose(bmf);
  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_BMF_Text(HWND hwnd)
{
bfFont far *fnt1;
bfFont far *fnt2;
bfFont far *fnt3;
HDC scrhdc;
HCURSOR hcurSave;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  if ((fnt1=bfActOpen("ARIAL032.BMF")) == bfNULL) {
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font 1", "Error", MB_OK);
    return;
  }
  if ((fnt2=bfActOpen("ARIIT032.BMF")) == bfNULL) {
    bfActClose(fnt1);
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font 2", "Error", MB_OK);
    return;
  }
  if ((fnt3=bfActOpen("ARIB1032.BMF")) == bfNULL) {
    bfActClose(fnt2);
    bfActClose(fnt1);
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font 3", "Error", MB_OK);
    return;
  }

  BackgroundDraw(scrhdc, 1);
  bfSetFont(bfNULL, fnt1);
  bfSetColors(bfNULL, 48, 0, bfYELLOW, bfDARKGRAY, 1, bfBETA, bfGAMMA);
  bfSetDrawMode(bfNULL, bfPUT, 128);
  bfStrDraw(bfNULL, 60, 40, "This is (the fastest) PUT mode of text");
  bfSetFont(bfNULL, fnt2);
  bfSetDrawMode(bfNULL, bfBLEND, 128);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfBLACK, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 60, 110, "This is just a plain old test of text,");
  bfStrDraw(bfNULL, 60, 150, "using BLEND mode.  BLEND offers no ");
  bfStrDraw(bfNULL, 60, 190, "transparency, but is fastest when the");
  bfStrDraw(bfNULL, 60, 230, "background is not solid.  Edges are");
  bfStrDraw(bfNULL, 60, 270, "blended for best anti-aliased quality.");
  bfSetFont(bfNULL, fnt3);
  bfSetDrawMode(bfNULL, bfTRANS, 110);
  bfStrDraw(bfNULL, 60, 330, "Now we've moved into TRANS mode,");
  bfStrDraw(bfNULL, 60, 370, "so that we can see the slowdown");
  bfSetColors(bfNULL, 48, 0, bfCYAN, bfBLACK, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 60, 410, "associated with the extra work.");

  bfActClose(fnt3);
  bfActClose(fnt2);
  bfActClose(fnt1);
  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_BMF_Box(HWND hwnd)
{
bfFont *bmf;
HDC scrhdc;
int Xcenter=320;
HCURSOR hcurSave;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  if ((bmf=bfActOpen("ARIAL032.BMF")) == bfNULL) {
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);
    return;
  }

  BackgroundDraw(scrhdc, 1);
  bfSetFont(bfNULL, bmf);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfCYAN, 1, bfBETA, bfGAMMA);
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetDrawMode(bfNULL, bfBLEND|bfTRANS, 128);
  bfSetUnderline(bfNULL, 3, 3);
  bfSetBackMode(bfNULL, bfPUT, 0);

  bfSetJust(bfNULL, bfHCENTER, bfVTOP);

  bfSetBackStyle(bfNULL, bfBACKSOLID, 1);
  bfSetBackSize(bfNULL, 0,0,0,0);
  bfSetFace(bfNULL, bfNORMAL);
  bfSetDrawMode(bfNULL, bfBLEND, 0);
  bfStrDraw(bfNULL, Xcenter, 10, "Solid text color here");
  bfSetFace(bfNULL, bfUNDER);
  bfSetDrawMode(bfNULL, bfTRANS, 110);
  bfStrDraw(bfNULL, Xcenter, 44, "Transparent, with underline");
  bfSetFace(bfNULL, bfDROP);
  bfSetDropMode(bfNULL, bfTRANS, 110);
  bfSetDrawMode(bfNULL, bfBLEND, 0);
  bfSetDropColors(bfNULL, 145, bfCYAN, bfCYAN, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, Xcenter, 84, "Transparent cyan dropshadow");
  bfSetBackMode(bfNULL, bfTRANS, 0);
  bfSetDropMode(bfNULL, bfBLEND, 0);
  bfSetDropColors(bfNULL, 220, bfBLACK, bfCYAN, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, Xcenter, 124, "0% background test");

  bfSetBackStyle(bfNULL, bfBACKBORDER, 1);
  bfSetBackSize(bfNULL, 35,10,35,10);
  bfSetBackMode(bfNULL, bfTRANS, 64);
  bfSetFace(bfNULL, bfBACK);
  bfSetDrawMode(bfNULL, bfBLEND, 0);
  bfStrDraw(bfNULL, Xcenter, 180, "25% back, border 1.  Back bytes >64K.  j");
  bfSetFace(bfNULL, bfUNDER|bfBACK);
  bfSetBackStyle(bfNULL, bfBACKSOLID, 1);
  bfSetBackSize(bfNULL, 35,10,0,0);
  bfSetBackMode(bfNULL, bfTRANS, 128);
  bfSetDrawMode(bfNULL, bfTRANS, 120);
  bfStrDraw(bfNULL, Xcenter, 240, "50% back, with underline.  j");
  bfSetFace(bfNULL, bfDROP|bfBACK|bfUNDER);
  bfSetBackStyle(bfNULL, bfBACKSOLID, 1);
  bfSetBackSize(bfNULL, 0,0,35,10);
  bfSetBackMode(bfNULL, bfTRANS, 160);
  bfSetDropMode(bfNULL, bfTRANS, 85);
  bfSetDrawMode(bfNULL, bfBLEND, 0);
  bfSetDropColors(bfNULL, 160, bfBLUE, bfCYAN, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, Xcenter, 290, "60% background.  j");
  bfSetFace(bfNULL, bfDROP|bfBACK);
  bfSetBackMode(bfNULL, bfTRANS, 255);
  bfSetBackStyle(bfNULL, bfBACKBORDER, 2);
  bfSetBackSize(bfNULL, 20,10,20,10);
  bfSetDropMode(bfNULL, bfBLEND, 0);
  bfSetDropColors(bfNULL, 240, bfBLACK, bfCYAN, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, Xcenter, 360, "100% back, border 2.  j");

  bfSetJust(bfNULL, bfHLEFT, bfVBASE);
  bfSetFace(bfNULL, bfNORMAL);
  bfSetBackMode(bfNULL, bfPUT, 0);
  bfSetBackStyle(bfNULL, bfBACKSOLID, 1);
  bfSetBackSize(bfNULL, 0,0,0,0);

  bfActClose(bmf);
  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_BMF_Frost(HWND hwnd)
{
bfFont *bmf;
HDC scrhdc;
int Xleft=18;
HCURSOR hcurSave;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

  if ((bmf=bfActOpen("ARIAL032.BMF")) == bfNULL) {
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);
    return;
  }
  Rectangle(scrhdc, 0,0, 32000,32000);

  bfSetFont(bfNULL, bmf);
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetDrawMode(bfNULL, bfPUT, 0);
  /*  Do this text in blue of course!!  */
  bfSetColors(bfNULL, 48, 0, RGB(255,0,0), bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, Xleft, 90, "Why make so much of fragmentary blue");
  bfStrDraw(bfNULL, Xleft, 140, "In here or there a bird, or butterfly,");
  bfStrDraw(bfNULL, Xleft, 190, "Or flower, or wearing stone, or open eye");
  bfStrDraw(bfNULL, Xleft, 240, "When heaven presents in sheets the solid hue?");
  bfStrDraw(bfNULL, Xleft+300, 310, "> Robert Frost");
  bfActClose(bmf);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_TT_RGB(HWND hwnd)
{
bfFont *bmf;
HDC scrhdc;
LOGFONT lf;
HCURSOR hcurSave;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  BackgroundDraw(scrhdc, 1);

  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(32*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)"Times New Roman");
  if ((bmf=bfActOpenTT(scrhdc, (LPLOGFONT)&lf, "", bfBOTH)) == bfNULL) {
    SetCursor(hcurSave);
    ReleaseDC(hwnd, scrhdc);
    MessageBox(NULL, "Couldn't load font", "Error", MB_OK);
    return;
  }
  bfSetFont(bfNULL, bmf);

  bfSetDrawMode(bfNULL, bfBLEND, 0);
  bfSetColors(bfNULL,0,0, bfWHITE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,45,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfLIGHTGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,80,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,115,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfDARKGRAY, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,150,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfBLACK, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,185,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfRED, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,220,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfGREEN, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,255,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfBLUE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,290,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfYELLOW, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,325,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfCYAN, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,360,"Text Line For Test!");
  bfSetColors(bfNULL,0,0, bfPURPLE, bfWHITE, 1, bfBETA, bfGAMMA);
  bfStrDraw(bfNULL, 160,395,"Text Line For Test!");

  bfActClose(bmf);
  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_TT_ShowAll(HWND hwnd)
{
HDC scrhdc;
HCURSOR hcurSave;
HPEN hpen, hpenOld;
HBRUSH hbr, hbrOld;
LOGFONT lf;
bfFont far *bmf;
int mypoints=24, x, y;
char font1[]="Times New Roman";

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfDARKGRAY, 1, bfBETA, bfGAMMA);
  bfSetDrawMode(bfNULL, bfBLEND, 0);

  /*  Draw the background.  */
  hpen = CreatePen(PS_SOLID, 0, RGB(64, 64, 64));
  hpenOld = (HPEN)SelectObject(scrhdc, hpen);
  hbr = CreateSolidBrush(RGB(64, 64, 64));
  hbrOld = (HBRUSH)SelectObject(scrhdc, hbr);
  Rectangle(scrhdc, 0,0, 32000,32000);
  SelectObject(scrhdc, hpenOld);
  SelectObject(scrhdc, hbrOld);
  DeleteObject(hpen);
  DeleteObject(hbr);

  /*  Draw the larger test font.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, (LPLOGFONT)&lf, "", bfBOTH)) != bfNULL) {
    for (y=0; y<6; y++) {        /* we're skipping 0-31.. */
      bfGfxRect(bfPUT, bfBACKSOLID, 0, bfBLACK, 0,
                20,y*34+32-bfGetFontAscent(bmf),
                620,y*34+32+bfGetFontDescent(bmf)-1);
      bfGfxLine(bfDARKGRAY, 0, 20,y*34+32, 620,y*34+32);
      for (x=0; x<16; x++)
        bfChrDraw(bfNULL, x*34+32, y*34+32, y*16+x+32, 0);
    }
    bfActClose(bmf);
  }

  /*  Draw the smaller test font.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints/2*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, (LPLOGFONT)&lf, "", bfBOTH)) != bfNULL) {
    for (y=0; y<6; y++) {        /* we're skipping 0-31.. */
      bfGfxRect(bfPUT, bfBACKSOLID, 0, bfBLACK, 0,
                20,y*34+32-bfGetFontAscent(bmf)+222,
                620,y*34+32+bfGetFontDescent(bmf)-1+222);
      bfGfxLine(bfDARKGRAY, 0, 20,y*34+32+222, 620,y*34+32+222);
      for (x=0; x<16; x++)
        bfChrDraw(bfNULL, x*34+32, y*34+32+222, y*16+x+32, 0);
    }
    bfActClose(bmf);
  }

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_TT_Alphabet(HWND hwnd)
{
HDC scrhdc;
LOGFONT lf;
HCURSOR hcurSave;
bfFont far *bmf;
int mypoints=42, PosX=70, PosY=45;
double gamma=0.8;
char fontname[]="Times New Roman";

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  BackgroundDraw(scrhdc, 1);

  /*  Draw the first alphabet.  */
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfBLACK, 1, bfBETA, gamma);
  bfSetDrawMode(bfNULL, bfBLEND, 0);
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)fontname);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    bfStrDraw(bfNULL, PosX, PosY,     "ABCDEFGHIJKLM");
    bfStrDraw(bfNULL, PosX, PosY+45,  "NOPQRSTUVWXYZ");
    bfStrDraw(bfNULL, PosX, PosY+95,  "abcdefghijklm");
    bfStrDraw(bfNULL, PosX, PosY+140, "nopqrstuvwxyz");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  /*  Draw the second alphabet.  */
  bfSetDrawMode(bfNULL, bfTRANS, 110);
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  mypoints-=2;  /* go bigger */
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)fontname);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    bfStrDraw(bfNULL, PosX, PosY+195, "ABCDEFGHIJKLM");
    bfStrDraw(bfNULL, PosX, PosY+250, "NOPQRSTUVWXYZ");
    bfStrDraw(bfNULL, PosX, PosY+305, "abcdefghijklm");
    bfStrDraw(bfNULL, PosX, PosY+360, "nopqrstuvwxyz");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_TT_Compare(HWND hwnd)
{
HDC scrhdc;
LOGFONT lf;
HFONT hMyFont;
HFONT hfOld;
HCURSOR hcurSave;
bfFont far *bmf;
int mypoints=36, PosX=20, PosY=30;
double gamma=0.8;
char font1[]="Times New Roman";

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  Rectangle(scrhdc, 0,0, 32000,32000);
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL, 48, 0, bfBLACK, bfWHITE, 1, bfBETA, gamma);
  bfSetDrawMode(bfNULL, bfPUT, 0);

  /*  Print the regular TrueType font.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
    TextOut(scrhdc, PosX, PosY, "1- This is a sample for testing", 31);
    SelectObject(scrhdc, hfOld);
    DeleteObject(hMyFont);
  }
  else
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);

  /*  Do the anti-aliased sample.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    bfStrDraw(bfNULL, PosX, PosY+100, "1- This is a sample for testing");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  /*  Print the regular TrueType font.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
    TextOut(scrhdc, PosX, PosY+130, "2- This is a sample for testing", 31);
    SelectObject(scrhdc, hfOld);
    DeleteObject(hMyFont);
  }
  else
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);

  /*  Do the anti-aliased sample.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    bfStrDraw(bfNULL, PosX, PosY+230, "2- This is a sample for testing");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  /*  Print the regular TrueType font.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(15, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
    TextOut(scrhdc, PosX+20, PosY+270, "3- This is a sample for testing", 31);
    SelectObject(scrhdc, hfOld);
    DeleteObject(hMyFont);
  }
  else
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);

  /*  Do the anti-aliased sample.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(15*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    bfStrDraw(bfNULL, PosX+20, PosY+310, "3- This is a sample for testing");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_TT_Large(HWND hwnd)
{
HDC scrhdc;
LOGFONT lf;
HFONT hMyFont;
HFONT hfOld;
HCURSOR hcurSave;
bfFont far *bmf;
int mypoints=180;
char fontname[]="Times New Roman";

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  Rectangle(scrhdc, 0,0, 32000,32000);

  /*  Print the Loading message.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(25, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)"Times New Roman");
  if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
    TextOut(scrhdc, 50,100, "Loading the large TrueType font..", 33);
    SelectObject(scrhdc, hfOld);
    DeleteObject(hMyFont);
  }

  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfBLACK, 1, bfBETA, bfGAMMA);
  bfSetDrawMode(bfNULL, bfBLEND, 0);

  /*  Print the Large.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)fontname);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    BackgroundDraw(scrhdc, 1);
    bfStrDraw(bfNULL, 20, 300, "Large");
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_CustomText(HWND hwnd)
{
HDC scrhdc;
LOGFONT lf;
HFONT hMyFont;
HFONT hfOld;
HCURSOR hcurSave;
bfFont far *bmf;
int mypoints, x, y;
char fontname[]="Times New Roman";
static char str1[128]="aka: Doc Raster";
static char str2[128]="64";

  /*  Get the custom information.  */
  if (Custom(hwnd, str1, str2) == IDCANCEL)
    return;
  mypoints = atoi(str2);

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  x = GetDeviceCaps(scrhdc, HORZRES) / 2;
  y = GetDeviceCaps(scrhdc, VERTRES) / 2;
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

  /*  Print the Loading message.  */
  if (mypoints > 40) {
    Rectangle(scrhdc, 0,0, 32000,32000);
    _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
    lf.lfHeight = -MulDiv(25, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
    lf.lfWeight = FW_DONTCARE;
    lf.lfItalic = 1;
    _fstrcpy((char far *)lf.lfFaceName, (char far *)"Times New Roman");
    if ((hMyFont = CreateFontIndirect(&lf)) != NULL) {
      hfOld = (HFONT)SelectObject(scrhdc, hMyFont);
      TextOut(scrhdc, 50,100, "Loading the TrueType font..", 27);
      SelectObject(scrhdc, hfOld);
      DeleteObject(hMyFont);
    }
  }

  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  bfSetColors(bfNULL, 48, 0, bfWHITE, bfBLACK, 1, bfBETA, bfGAMMA);
  bfSetDrawMode(bfNULL, bfTRANS, 136);

  /*  Print the Text.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints*4, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 1;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)fontname);
  if ((bmf=bfActOpenTT(scrhdc, &lf, "", bfBOTH)) != bfNULL) {
    BackgroundDraw(scrhdc, 1);
    bfSetJust(bfNULL, bfHCENTER, bfVCENTER);
    bfStrDraw(bfNULL, x, y, str1);
    bfSetJust(bfNULL, bfHLEFT, bfVBASE);
    bfActClose(bmf);
  }
  else
    MessageBox(NULL, bfGetErrorString(bfGetError(bfCLEAR)), "Error", MB_OK);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_GFX_Rect(HWND hwnd)
{
HDC scrhdc;
HCURSOR hcurSave;
int x1=25, x2=615, y1=54, y2=104;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);
  BackgroundDraw(scrhdc, 2);

  bfGfxRect((bfDrawMode)(bfTRANS+(100<<8)),
            bfBACKSOLID, 0, bfCYAN, 0, x1,y1, x2,y2);
  y1 += 70;
  y2 += 70;
  bfGfxRect((bfDrawMode)(bfTRANS+(100<<8)),
            bfBACKSOLID, 0, bfBLACK, 0, x1,y1, x2,y2);
  y1 += 70;
  y2 += 70;
  bfGfxRect(bfPUT, bfBACKBORDER, 1, bfCYAN, 0, x1,y1, x2,y2);
  y1 += 73;
  y2 += 77;
  bfGfxRect((bfDrawMode)(bfTRANS+(90<<8)),
            bfBACKBORDER, 5, bfBLACK, 0, x1,y1, x2,y2);

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


void cm_GFX_Zoom(HWND hwnd)
{
HDC scrhdc;
HCURSOR hcurSave;
HPEN hpen, hpenOld;
HBRUSH hbr, hbrOld;
HFONT hf, hfOld;
LOGFONT lf;
bfFont far *bmf;
bfBMFFontChar bfFAR *c;
char testchars[]="VWXY_vwxy";
int baseline2=300, X=5, mypoints=42, scale=8, t=0, x,y, clr;
char font1[]="Times New Roman";
unsigned char far *pixels;

  if ((scrhdc=GetDC(hwnd)) == NULL) {
    MessageBox(NULL, "Couldn't get hDC", "Error", MB_OK);
    return;
  }
  hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
  bfSetBitMap(bfNULL, (bfBitMapPtr)scrhdc);

  /*  Draw this background.  */
  hpen = CreatePen(PS_SOLID, 0, RGB(0, 0, 0));
  hpenOld = (HPEN)SelectObject(scrhdc, hpen);
  hbr = CreateSolidBrush(RGB(0, 0, 0));
  hbrOld = (HBRUSH)SelectObject(scrhdc, hbr);
  Rectangle(scrhdc, 0,0, 32000,32000);
  SelectObject(scrhdc, hpenOld);
  SelectObject(scrhdc, hbrOld);
  DeleteObject(hpen);
  DeleteObject(hbr);
  hpen = CreatePen(PS_SOLID, 0, RGB(200, 50, 50));
  hpenOld = (HPEN)SelectObject(scrhdc, hpen);
  MoveTo(scrhdc, 0,baseline2);
  LineTo(scrhdc, 639,baseline2);
  SelectObject(scrhdc, hpenOld);
  DeleteObject(hpen);

  /*  Draw the regular TrueType.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((hf = CreateFontIndirect(&lf)) != NULL) {
    hfOld = (HFONT)SelectObject(scrhdc, hf);
    TextOut(scrhdc, 70, 150, testchars, strlen(testchars));
    SelectObject(scrhdc, hfOld);
    DeleteObject(hf);
  }
  else
    MessageBox(NULL, "Couldn't open font", "Error", MB_OK);

  /*  Draw the zoomed anti-aliased image.  */
  _fmemset((char far *)&lf, 0, sizeof(LOGFONT));
  lf.lfHeight = -MulDiv(mypoints, GetDeviceCaps(scrhdc, LOGPIXELSY), 72);
  lf.lfWidth = 0;
  lf.lfItalic = 0;
  lf.lfWeight = FW_DONTCARE;
  _fstrcpy((char far *)lf.lfFaceName, (char far *)font1);
  if ((bmf=bfActOpenTT(scrhdc, (LPLOGFONT)&lf, "", bfBOTH)) != bfNULL) {
    while (testchars[t] != 0) {
      c = &bmf->data.BMFFont.chars[testchars[t]];
      pixels = c->pixels;
      for (y=c->boxY-1; y>=0; y--) {
        for (x=0; x<c->boxX; x++) {
          if (*pixels) {
            clr = (*pixels << 4) | *pixels;
            hpen = CreatePen(PS_SOLID, 0, RGB(clr, clr, clr));
            hpenOld = (HPEN)SelectObject(scrhdc, hpen);
            hbr = CreateSolidBrush(RGB(clr, clr, clr));
            hbrOld = (HBRUSH)SelectObject(scrhdc, hbr);
            Rectangle(scrhdc,
              X+(x*scale),       baseline2-(y*scale)-scale+(c->boxDescent*scale),
              X+(x*scale)+scale, baseline2-(y*scale)+(c->boxDescent*scale) );
            SelectObject(scrhdc, hpenOld);
            SelectObject(scrhdc, hbrOld);
            DeleteObject(hpen);
            DeleteObject(hbr);
          }
          pixels++;
        }
      }
      t++;
      X += c->boxX*scale;
    }
    bfActClose(bmf);
  }

  SetCursor(hcurSave);
  ReleaseDC(hwnd, scrhdc);
}


long FAR PASCAL WndProc( HWND hwnd, WORD message, WORD wParam, LONG lParam)
{
int loops=1;

  switch (message) {
  case WM_COMMAND:
    switch (wParam) {
    case CM_ABOUT:
      cm_About(hwnd);
      return 0;
    case CM_ABOUTCLOSE:
      DestroyWindow(hwnd);
      break;
    case CM_BMF_RGB:
      cm_BMF_RGB(hwnd);
      return 0;
    case CM_BMF_TEXT:
      cm_BMF_Text(hwnd);
      return 0;
    case CM_BMF_BOX:
      cm_BMF_Box(hwnd);
      return 0;
    case CM_BMF_FROST:
      cm_BMF_Frost(hwnd);
      return 0;
    case CM_TT_RGB:
      cm_TT_RGB(hwnd);
      return 0;
    case CM_TT_SHOWALL:
      cm_TT_ShowAll(hwnd);
      return 0;
    case CM_TT_ALPHA:
      cm_TT_Alphabet(hwnd);
      return 0;
    case CM_TT_COMPARE:
      cm_TT_Compare(hwnd);
      return 0;
    case CM_TT_LARGE:
      cm_TT_Large(hwnd);
      return 0;
    case CM_BMF_LOOPTEST:
      if (Loops(hwnd, &loops) == IDOK) {
        while (loops-- > 0) {
          cm_BMF_RGB(hwnd);
          cm_BMF_Text(hwnd);
          cm_BMF_Box(hwnd);
          cm_BMF_Frost(hwnd);
        }
      }
      return 0;
    case CM_TT_LOOPTEST:
      if (Loops(hwnd, &loops) == IDOK) {
        while (loops-- > 0) {
          cm_TT_RGB(hwnd);
          cm_TT_ShowAll(hwnd);
          cm_TT_Alphabet(hwnd);
          cm_TT_Compare(hwnd);
          cm_TT_Large(hwnd);
        }
      }
      return 0;
    case CM_LOOPBOTH:
      if (Loops(hwnd, &loops) == IDOK) {
        while (loops-- > 0) {
          cm_BMF_RGB(hwnd);
          cm_BMF_Text(hwnd);
          cm_BMF_Box(hwnd);
          cm_BMF_Frost(hwnd);
          cm_TT_RGB(hwnd);
          cm_TT_ShowAll(hwnd);
          cm_TT_Alphabet(hwnd);
          cm_TT_Compare(hwnd);
          cm_TT_Large(hwnd);
        }
      }
      return 0;
    case CM_CUSTOMTEXT:
      cm_CustomText(hwnd);
      return 0;
    case CM_GFX_RECT:
      cm_GFX_Rect(hwnd);
      return 0;
    case CM_GFX_ZOOM:
      cm_GFX_Zoom(hwnd);
      return 0;
    }
    break;
  case WM_CREATE:
    bfActInit(bf24BIT);                  /*  Init the font DLL.     */
    break;
  case WM_DESTROY:
    {
    int fonts_freed = bfActEnd();        /*  Un-init the font DLL.  */
    if (fonts_freed > 0) {
      char s[80];
      sprintf(s, "bfActEnd() closed %d font(s) automatically.",fonts_freed);
      MessageBox(NULL, s, "BeautiFonts (tm) Demo", MB_OK);
    }
    BackgroundDraw((HDC)0, 0);           /*  Free background BMPs.   */
    PostQuitMessage(0);
    return 0;
    }
  }
  return DefWindowProc(hwnd, message, wParam, lParam);
}


#pragma argsused
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszCmdParam, int nCmdShow)
{
static char szAppName[] = "BFDEMO";
MSG         msg;
WNDCLASS    wndclass;
HWND        Hwnd;

  #if (!bfNOMEMCHECK)
    mc_startcheck(NULL);       /*  Start MemCheck for BFDEMO  */
    bfMemCheck_Start();        /*  Start MemCheck for BFDLL   */
  #endif

  if (!hPrevInstance) {
    wndclass.style          = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc    = (WNDPROC) WndProc;
    wndclass.cbClsExtra     = 0;
    wndclass.cbWndExtra     = 0;
    wndclass.hInstance      = hInstance;
    wndclass.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
    wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wndclass.lpszMenuName   = "COMMANDS";
    wndclass.lpszClassName  = szAppName;
    RegisterClass(&wndclass);
  }
  /*  else { init app's instance? }  */

  /*  Create an instance of the new window class.  */
  Hwnd = CreateWindow( szAppName,
         "BeautiFonts Demo.  Copyright \251 1995 Brian Reed, All Rights Reserved.",
          WS_OVERLAPPEDWINDOW,
          0,0, 640, 480, NULL, NULL, hInstance, NULL );
  ShowWindow(Hwnd, nCmdShow);
  UpdateWindow(Hwnd);

  /*  Standard Windows polling and exit.  */
  while ( GetMessage(&msg, NULL, 0,0) ) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  #if (!bfNOMEMCHECK)
    bfMemCheck_End();          /*  Close MemCheck for BFDLL   */
    mc_endcheck();             /*  Close MemCheck for BFDEMO  */
  #endif

  return msg.wParam;
}


